
/*
 * $Id: SrvOpts.cc,v 1.6 1999/05/03 23:17:09 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include "SharedOpts.h"
#include "SrvOpts.h"

#include "rndDistrs.h"
#include "globals.h"

SrvOpts TheSrvOpts;


SrvOpts::SrvOpts():
	thePort(this,        "port <int>",         "listen port number"),
	theAcceptLmt(this,   "accept_lmt <int>",   "accept no more at once"),
	theTimeout(this,     "idle_tout <time>",   "exit after inactivity timeout"),
	theRepSize(this,     "rep_sz <size>",      "mean reply cont. length", new ExpDistr(new RndGen, Size::KB(13))),
	trackRPorts(this,    "track_rports <bool>","track remote port [re]use"),
	theMinRPortWait(this,"rport_wait_min <time>","min a remote port should be in TIME_WAIT", Time::Sec(60)),

	theObjBday(this,     "obj_bday <timeDistr>","object creation time (-rel or +abs)", new UnifDistr(new RndGen, Time::Yeard(10).secd(), Time::Yeard(20).secd())),
	theModTime(this,     "obj_life_cycle <timeDistr>","how often an object gets modified"),
	theModTimeVar(this,  "obj_life_cycle_var <%>","how variable obj mod times are", 0),
	theShowModRatio(this,"obj_with_lmt <%>",   "ratio of objects with Last-Modified: header", 0),
	theObjLifeCycle(this,"obj_expire <ObjExp>","various object expiration models")
{
	theRepSize.argType("size");
	theObjBday.argType("time");
	theModTime.argType("time");
}

bool SrvOpts::validate() const {
	if (thePort < 0)
		cerr << ThePrgName << ": no listen port specified" << endl;
	else
	if (!theRepSize)
		cerr << ThePrgName << ": rep size distribution must be set" << endl;
	else
	if (theMinRPortWait.secd() > theMinRPortWait.sec())
		cerr << ThePrgName << ": fractional seconds are not suported in rport_wait_min; got: " << theMinRPortWait << endl;
	else
	if (!theModTime && theShowModRatio > 0)
		cerr << ThePrgName << ": must specify object life cycle to generate Last-Modified: header fields" << endl;
	else
	if (!theModTime && (theObjLifeCycle.modExpTime() || theObjLifeCycle.nowExpTime()))
		cerr << ThePrgName << ": must specify object life cycle to use obj expiration model" << endl;
	else
	if (theModTime && !theObjBday)
		cerr << ThePrgName << ": must specify object birthday when obj life cycle is in use" << endl;
	else
		return OptGrp::validate();
	return false;
}
