
/*
 * $Id: Xaction.cc,v 1.22 1999/06/02 06:26:35 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include "Connection.h"
#include "Clock.h"
#include "StatPing.h"
#include "StatPhaseMgr.h"
#include "Progress.h"
#include "SharedOpts.h"
#include "Xaction.h"
#include "polyErrors.h"
#include "globals.h"

void Xaction::reset() {
	theConn = 0;
	theId.clear();
	theStartTime = theLifeTime = Time();
	theXactClass = -1;
	theState = stNone;
}

void Xaction::start(Connection *aConn) {
	Assert(aConn);
	theConn = aConn;

	theStartTime = theConn->useStart();
	CallStat(ThePinger, xactBeg());
}

void Xaction::stop(int errNo) {
	Assert(theConn);
	if (theState == stSpaceWaiting)
		theConn->theWr.stop(this);
	else
		theConn->theRd.stop(this);
	
	theLifeTime = TheClock - theStartTime;

	if (!errNo) {
		stopStats();
	} else {
		theConn->bad(true);
		// low level errors should be reported when they occur
		// low level errors are propogated to us as "-1"
		// do not count/report them twice
		if (errNo > 0)
			ReportError(errNo);
		CallStat(ThePinger, xactErr());
		TheProgress.theErrs.theCnt++;
	}

	theState = stDone;
}

void Xaction::stopStats() {
	CallStat(TheSoOftenPinger, xactLifeTime(theLifeTime, theXactClass));
	TheProgress.theXactComplCnt++;
	CallStat(ThePinger, xactEnd());
}
