
/*
 * $Id: NetAddr.h,v 1.2 1999/04/25 06:32:13 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef X_NET_ADDR_H
#define X_NET_ADDR_H

#include <sys/types.h>
#include <netinet/in.h>

// this is needed on Solaris, any other?
#ifndef INADDR_NONE
#ifndef _IN_ADDR_T
#define _IN_ADDR_T
typedef unsigned int    in_addr_t;
#endif
#define INADDR_NONE ((in_addr_t)-1)
#endif

#include "String.h"

// inet_addr wrapper which also stores port and
// does _lazy_ ntoa and aton maipulations
class NetAddr {
	public:
		NetAddr(): thePort(-1) { theAddrN.s_addr = INADDR_NONE; }
		NetAddr(const String &addr, int aPort): theAddrA(addr), thePort(aPort) { theAddrN.s_addr = INADDR_NONE; }
		NetAddr(in_addr addr, int aPort): theAddrN(addr), thePort(aPort) {}

		operator bool() const { return thePort >= 0; }
		bool operator == (const NetAddr &addr) const;
		bool operator != (const NetAddr &addr) const { return !(*this == addr); }

		const String &addrA() const { if (!theAddrA) syncA(); return theAddrA; }
		in_addr addrN() const { if (theAddrN.s_addr == INADDR_NONE) syncN(); return theAddrN; }
		int port() const { return thePort; }

		void addr(const String &addr) { theAddrA = addr; theAddrN.s_addr = INADDR_NONE; }
		void addr(in_addr addr) { theAddrN = addr; theAddrA = 0; }
		void port(int aPort) { thePort = aPort; }

		ostream &print(ostream &os) const;

		//bool fqdn(String &name);

	protected:
		void syncA() const; // theAddrA <- theAddrN
		void syncN() const; // theAddrN <- theAddrA

	protected:
		mutable String theAddrA;  // FQDN or IP; set if not null
		mutable in_addr theAddrN; // IP if available; set if >= 0;
		int thePort;              // set if >= 0;
};

inline ostream &operator <<(ostream &os, const NetAddr &addr) { return addr.print(os); }

#endif
