
/*
 * $Id: Poll.h,v 1.3 1999/06/08 03:41:05 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef XSTD_POLL_H
#define XSTD_POLL_H

#include "fixPoll.h"

#include "Array.h"
#include "FileScanner.h"

// Poll is a File Scanner based on poll(2) system call

// aggregate bit masks together (or not)
#define XPOLL_RD POLLRDNORM
#define XPOLL_WR POLLWRNORM
#define XPOLL_EX (POLLERR | POLLHUP | POLLNVAL)

// pollfd wrapper
// note: optimized for efficient copying and such
class PollFD: public pollfd {
	public:
		PollFD(int aFD = -1);

		void init(int aFD); // must be called to initialize the object

		bool used() const { return events != 0; }
		inline bool ready(IODir dir) const;
		
		void set(IODir dir);
		void clear(IODir dir);
};

// pollfd + user
class Pollee: public PollFD {
	typedef FileScanUser User;
	public:
		Pollee(int aFD = -1);
		
		void set(IODir dir, User *u);
		void clear(IODir dir);

		User *user() { return theUser; }

	protected:
		User *theUser;
};


class Poll: public FileScanner {
	public:
		typedef FileScanUser User;

	public:
		Poll();

		virtual const char *name() const { return "poll"; }

		virtual void configure(int fdLimit);

		virtual FileScanReserv setFD(int fd, IODir dir, User *p);
		virtual void clearFD(int fd, IODir dir);

	protected:
		virtual int sweep(Time *timeout = 0);
		virtual User *readyUser(int fd, IODir dir, int &fd);

	protected:
		Array<Pollee> thePollees;

	private:
		Array<PollFD> theHotFDs; // used during poll call only
		int theMaxFD;
};


/* inlined methods */

inline
bool PollFD::ready(IODir dir) const {
	// note: ready means "listening" and "has an event"
	if (dir == dirRead)
		return (events & XPOLL_RD) && (revents & (XPOLL_RD | XPOLL_EX));
	else
		return (events & XPOLL_WR) && (revents & (XPOLL_WR | XPOLL_EX));
}

#endif
