
/*
 * $Id: Select.h,v 1.2 1999/05/22 03:09:30 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef XSTD_SELECT_H
#define XSTD_SELECT_H

#include <sys/types.h>
#include <unistd.h>

#include "Array.h"
#include "FileScanner.h"

// Select is a File Scanner based on select(2) system call


// a wrapper arround fd_set
class FD_Set {
	typedef FileScanUser User;
	public:
		FD_Set();
		~FD_Set();

		void fdLimit(int limit);

		int maxFD() const { return theMaxFD; }
		int resCount() const { return theResCount; }
		bool active() const { return theMaxFD >= 0; }
		bool readyActive() const { return theReadyMaxFD >= 0; }
		fd_set *makeReadySet() { theReadySet = theSet; theReadyMaxFD = theMaxFD; return readyActive() ? &theReadySet : 0; }

		void setFD(int fd, User *u);
		void clearFD(int fd);

		bool isSet(int fd) const { return FD_ISSET(fd, &theSet); }
		bool isReady(int fd) const { return FD_ISSET(fd, &theReadySet); }
		User *operator [](int fd) { return theUsers[fd]; }

		User *readyUser(int fd) { return isReady(fd) ? theUsers[fd] : 0; }

	protected:
		Array<User*> theUsers;
		fd_set theSet;
		fd_set theReadySet;

		int theMaxFD;
		int theReadyMaxFD;
		int theResCount;
};


class Select: public FileScanner {
	public:
		typedef FileScanUser User;

	public:
		Select();

		virtual const char *name() const { return "select"; }

		virtual void configure(int fdLimit);

		virtual FileScanReserv setFD(int fd, IODir dir, User *p);
		virtual void clearFD(int fd, IODir dir);

	protected:
		virtual int sweep(Time *timeout = 0);
		virtual User *readyUser(int idx, IODir dir, int &fd);

	protected:
		FD_Set theSets[2]; // read/write
		int theMaxFD;
};

#endif
