
/*
 * $Id: Socket.h,v 1.1.1.3 1999/02/22 06:19:53 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef X_SOCKET_CLASS_H
#define X_SOCKET_CLASS_H

#include <sys/types.h>
#include <sys/socket.h>

#include "Time.h"
#include "Size.h"
#include "NetAddr.h"

// UNIX socket wrapper
class Socket {
	public:
		static bool IsEnabled;  // bind,connect,listen,read,write calls are enabled
		static int TheMaxLevel; // will not allow Level to go higher
		static int Level() { return TheLevel; }

	public:
		Socket(): theFD(-1) {}
		Socket(int aFD): theFD(aFD) {}
		~Socket() {}

		bool create();
		bool create(int domain, int type, int protocol);
		bool connect(const NetAddr &addr);
		bool bind(int port);
		bool listen(int port);
		Socket accept(struct sockaddr *addr, int *addr_len);
		Size read(void *buf, Size sz);
		Size write(const void *buf, Size sz);
		bool close();

		bool reuseAddr(bool set);
		bool reusePort(bool set);
		bool linger(int secs);
		bool blocking(bool set);
		bool tcpDelay(bool set);
		bool sendBuf(Size size);
		bool recvBuf(Size size);

		int fd() const { return theFD; }
		bool isOpen() const { return theFD >= 0; }
		bool isBlocking() const;
		int flags() const;
		int lport() const;
		int rport() const;
		Time rcvTimeout() const;
		Size sendBuf() const;
		Size recvBuf() const;
		int errNo() const;
		bool peerAddr(struct sockaddr *addr, int &len) const;
		NetAddr peerAddr() const;

		operator void*() const { return theFD >= 0 ? (void*)-1 : (void*)0; }

	protected:
		bool setOpt(int optName, bool set);
		bool setOpt(int optName, const void *optVal, int optLen);
		bool getOpt(int optName, void *optVal, int *optLen) const;

	protected:
		int theFD;

		static int TheLevel; // number of open sockets
};

#endif
